from intelli.model.input.image_input import ImageModelInput
from intelli.wrappers.openai_wrapper import OpenAIWrapper
from intelli.wrappers.stability_wrapper import StabilityAIWrapper


class RemoteImageModel:
    supported_image_models = {
        "openai": OpenAIWrapper,
        "stability": StabilityAIWrapper,
    }

    def __init__(self, api_key, provider="openai"):
        if provider in self.supported_image_models:
            self.provider = self.supported_image_models[provider](api_key)
        else:
            supported_models = ", ".join(self.supported_image_models.keys())
            raise ValueError(f"The received provider {provider} not supported. Supported providers: {supported_models}")

    def generate_images(self, image_input):
        if isinstance(image_input, dict):
            inputs = image_input
        elif isinstance(image_input, ImageModelInput):
            inputs = image_input.get_openai_inputs() if isinstance(self.provider,
                                                                   OpenAIWrapper) else image_input.get_stability_inputs()
        else:
            raise ValueError("image_input must be an instance of ImageModelInput or a dictionary.")

        results = self.provider.generate_images(inputs)

        if isinstance(self.provider, OpenAIWrapper):
            return [data['url'] if 'url' in data else data['b64_json'] for data in results['data']]
        else:
            return [image_obj['base64'] for image_obj in results['artifacts']]




import unittest
from intelli.controller.remote_embed_model import RemoteEmbedModel
from intelli.model.input.embed_input import EmbedInput
import os
from dotenv import load_dotenv
load_dotenv()

class TestRemoteEmbedModel(unittest.TestCase):
    
    # Set up API keys for different providers
    OPENAI_API_KEY = os.getenv('OPENAI_API_KEY')
    MISTRAL_API_KEY = os.getenv('MISTRAL_API_KEY')
    GEMINI_API_KEY = os.getenv('GEMINI_API_KEY')

    def test_openai_embeddings(self):
        """Test retrieving embeddings from OpenAI."""
        if self.OPENAI_API_KEY is None:
            self.skipTest("OPENAI_API_KEY environment variable is not set.")
            
        provider = 'openai'
        model = RemoteEmbedModel(provider, self.OPENAI_API_KEY)
        embed_input = EmbedInput(["This is a test sentence for embeddings."])
        embed_input.set_default_values(provider)
        
        result = model.get_embeddings(embed_input)
        self.assertIn('data', result, "OpenAI response should contain 'data' field")

    def test_gemini_embeddings(self):
        """Test retrieving embeddings from Gemini."""
        if self.GEMINI_API_KEY is None:
            self.skipTest("GEMINI_API_KEY environment variable is not set.")
            
        provider = 'gemini'
        model = RemoteEmbedModel(provider, self.GEMINI_API_KEY)
        embed_input = EmbedInput(["Explore Gemini's API for embeddings."], "models/embedding-001")
        
        result = model.get_embeddings(embed_input)
        self.assertIsInstance(result['values'], list, "Gemini response should be a list of embeddings")
    
    def test_mistral_embeddings(self):
        """Test retrieving embeddings from Mistral."""
        if self.MISTRAL_API_KEY is None:
            self.skipTest("MISTRAL_API_KEY environment variable is not set.")
            
        provider = 'mistral'
        model = RemoteEmbedModel(provider, self.MISTRAL_API_KEY)
        embed_input = EmbedInput(["Mistral provides interesting insights."])
        embed_input.set_default_values(provider)
        
        result = model.get_embeddings(embed_input)
        # Assuming a similar response format for simplicity; adjust according to actual API
        self.assertIn('data', result, "Mistral response should contain 'data' field")

if __name__ == '__main__':
    unittest.main()


class Text2SpeechInput:
    Gender = {
        'FEMALE': 'FEMALE',
        'MALE': 'MALE'
    }

    def __init__(self, text, language="en-gb", gender="FEMALE", voice=None, model='tts-1', stream=True):
        self.text = text
        self.language = language.lower()
        # gender is in uppercase for consistency with Gender dictionary keys
        self.gender = gender.upper()
        self.voice = voice
        self.model = model
        self.stream = stream

    def get_google_input(self):
        params = {'text': self.text, 'languageCode': self.language}

        language_name_map = {
            "en-gb": "en-GB",
            "en": "en-GB",
            "tr-tr": "tr-TR",
            "tr": "tr-TR",
            "cmn-cn": "cmn-CN",
            "cn": "cmn-CN",
            "de-de": "de-DE",
            "de": "de-DE",
            "ar-xa": "ar-XA",
            "ar": "ar-XA",
        }

        gender_name_map = {
            "FEMALE": "A",
            "MALE": "B"
        }

        base_language = language_name_map.get(self.language, None)

        if base_language:
            params['name'] = f"{base_language}-Standard-{gender_name_map.get(self.gender, 'A')}"
            params['ssmlGender'] = self.gender
        else:
            raise ValueError(f"Unsupported language code: {self.language}")

        return params

    def get_openai_input(self):
        return {
            'input': self.text,
            'voice': self.voice,
            'model': self.model,
            'stream': self.stream
        }



import unittest
import os
from dotenv import load_dotenv
from intelli.wrappers.geminiai_wrapper import GeminiAIWrapper
import base64
load_dotenv()

class TestGeminiAIWrapper(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        api_key = os.getenv("GEMINI_API_KEY")
        assert api_key is not None, "GEMINI_API_KEY is not set."
        cls.wrapper = GeminiAIWrapper(api_key)
    
    def test_generate_content(self):
        params = {
            "contents": [{
                "parts": [{
                    "text": "Write a story about a magic backpack."
                }]
            }]
        }

        result = self.wrapper.generate_content(params)
        print('content sample result: ', result['candidates'][0]['content']['parts'][0]['text'][:100])
        self.assertIsNotNone(result['candidates'][0]['content']['parts'][0]['text'])
    
    def test_image_to_text(self):
        file_path = '../temp/test_image_desc.png' 

        try:
            with open(file_path, "rb") as image_file:
                image_data = base64.b64encode(image_file.read()).decode('utf-8')
            
            result = self.wrapper.image_to_text('describe the image', image_data, 'png')

            self.assertTrue('candidates' in result, "The result should have a 'candidates' field.")
            self.assertIsInstance(result['candidates'], list, "Expected 'candidates' to be a list.")
            self.assertGreater(len(result['candidates']), 0, "'candidates' list should not be empty.")
            
            generated_text = result['candidates'][0]['content']['parts'][0]['text']
            print('Gemini AI Image To Text Generation Test Result:\n', generated_text, '\n')

            # Assert the generated text is not empty
            self.assertTrue(generated_text, 'Gemini AI returned no results')

        except Exception as error:
            self.fail(f'Gemini AI Error: {error}')
    
    def test_get_embeddings(self):
        text = "Write a story about a magic backpack."
        params = {
            "model": "models/embedding-001",
            "content": {
                "parts": [{
                    "text": text
                }]
            }
        }

        result = self.wrapper.get_embeddings(params)
        print('embedding sample result: ', result['values'][:5])
        self.assertTrue('values' in result)
    
    def test_get_batch_embeddings(self):
        texts = ["Hello world", "Write a story about a magic backpack."]
        requests = [{
            "model": "models/embedding-001",
            "content": {
                "parts": [{"text": text}]
            }
        } for text in texts]

        result = self.wrapper.get_batch_embeddings({"requests": requests})
        print('batch embedding sample result: ', result[0]['values'][:5])
        self.assertGreater(len(result), 0, "No batch embedding results returned.")
    
if __name__ == "__main__":
    unittest.main()



import unittest
import os
import asyncio
from intelli.function.chatbot import Chatbot
from intelli.model.input.chatbot_input import ChatModelInput
from dotenv import load_dotenv
load_dotenv()

class TestChatbot(unittest.TestCase):
    def setUp(self):
        self.openai_api_key = os.getenv("OPENAI_API_KEY")
        self.gemini_api_key = os.getenv("GEMINI_API_KEY")
        self.mistral_api_key = os.getenv("MISTRAL_API_KEY")
        
        # Creating Chatbot instances for each AI model
        self.openai_bot = Chatbot(self.openai_api_key, "openai")
        self.gemini_bot = Chatbot(self.gemini_api_key, "gemini")
        self.mistral_bot = Chatbot(self.mistral_api_key, "mistral")
    
    def test_openai_chat(self):
        print('---- start openai ----')
        input = ChatModelInput("You are a helpful assistant.", "gpt-3.5-turbo")
        input.add_user_message("What is the capital of France?")
        
        response = self.openai_bot.chat(input)

        print('openai response: ', response)
        
        self.assertTrue(len(response) > 0, "OpenAI chat response should not be empty")

    def test_gemini_chat(self):
        print('---- start gemini ----')
        input = ChatModelInput("You are a helpful assistant.", "gemini-model")
        input.add_user_message("Describe a starry night.")
        
        response = self.gemini_bot.chat(input)
        
        print('gemini response: ', response)

        self.assertTrue(len(response) > 0, "Gemini chat response should not be empty")


    def test_mistral_chat(self):
        print('---- start mistral ----')
        input = ChatModelInput("You are a helpful assistant.", "mistral-tiny")
        input.add_user_message("Who is Leonardo da Vinci?")
        
        response = self.mistral_bot.chat(input)

        print('mistral response: ', response)
        
        self.assertTrue(len(response) > 0, "Mistral chat response should not be empty")
    
    def test_openai_stream(self):
        print('---- start openai stream ----')
        input = ChatModelInput("You are a helpful assistant.", "gpt-3.5-turbo")
        input.add_user_message("Tell me a story about a lion in the savanna.")

        # Use asyncio.run() to get the result of the coroutine
        full_text = asyncio.run(self._get_openai_stream(input))

        print('openai stream response: ', full_text)

        self.assertTrue(len(full_text) > 0, "OpenAI stream response should not be empty")

    async def _get_openai_stream(self, chat_input):
        
        full_text = ''
        
        for content in self.openai_bot.stream(chat_input):
            full_text += content
            print('content item: ', content) 
        
        return full_text
    
if __name__ == '__main__':
    unittest.main()


from abc import ABC, abstractmethod

from intelli.controller.remote_image_model import RemoteImageModel
from intelli.controller.remote_vision_model import RemoteVisionModel
from intelli.flow.input.agent_input import AgentInput, TextAgentInput, ImageAgentInput
from intelli.flow.types import AgentTypes
from intelli.function.chatbot import Chatbot
from intelli.model.input.chatbot_input import ChatModelInput
from intelli.model.input.image_input import ImageModelInput
from intelli.model.input.vision_input import VisionModelInput


class BasicAgent(ABC):

    @abstractmethod
    def execute(self, agent_input):
        pass


class Agent(BasicAgent):
    def __init__(self, agent_type, provider, mission, model_params, options=None):

        if agent_type not in AgentTypes._value2member_map_:
            raise ValueError("Incorrect agent type. Accepted types in AgentTypes.")

        self.type = agent_type
        self.provider = provider
        self.mission = mission
        self.model_params = model_params
        self.options = options

    def execute(self, agent_input: AgentInput):

        # Check the agent type and call the appropriate function
        if self.type == AgentTypes.TEXT.value:

            f_params = {key: value for key, value in self.model_params.items() if hasattr(ChatModelInput("test", None), key)}

            chat_input = ChatModelInput(self.mission, **f_params)

            chatbot = Chatbot(self.model_params['key'], self.provider, self.options)
            chat_input.add_user_message(agent_input.desc)
            result = chatbot.chat(chat_input)[0]
        elif self.type == AgentTypes.IMAGE.value:

            f_params = {key: value for key, value in self.model_params.items() if hasattr(ImageModelInput("test"), key)}

            image_input = ImageModelInput(prompt=self.mission + ": " + agent_input.desc, **f_params)

            image_model = RemoteImageModel(self.model_params['key'], self.provider)
            result = image_model.generate_images(image_input)[0]
        elif self.type == AgentTypes.VISION.value:
            vision_input = VisionModelInput(content=self.mission + ": " + agent_input.desc,
                                            image_data=agent_input.img,
                                            extension=self.model_params.get('extension', 'png'),
                                            model=self.model_params['model'])

            vision_model = RemoteVisionModel(self.model_params['key'], self.provider)
            result = vision_model.image_to_text(vision_input)
        else:
            raise ValueError(f"Unsupported agent type: {self.type}.")

        return result



import json
import requests

from intelli.config import config
from intelli.utils.conn_helper import ConnHelper


class GoogleAIWrapper:

    def __init__(self, api_key):
        self.api_key = api_key
        self.headers = {
            'Content-Type': 'application/json; charset=utf-8',
            'X-Goog-Api-Key': self.api_key,
        }
        self.api_speech_url = config['url']['google']['base'].format(config['url']['google']['speech']['prefix'])

    def generate_speech(self, params):
        url = self.api_speech_url + config['url']['google']['speech']['synthesize']['postfix']
        param = self.get_synthesize_input(params)

        response = requests.post(url, headers=self.headers, data=json.dumps(param))
        response.raise_for_status()
        return response.json()['audioContent']

    def get_synthesize_input(self, params):
        return {
            'input': {
                'text': params['text'],
            },
            'voice': {
                'languageCode': params['languageCode'],
                'name': params['name'],
                'ssmlGender': params['ssmlGender'],
            },
            'audioConfig': {
                'audioEncoding': 'MP3',
            },
        }



from intelli.utils.logging import Logger


class SequenceFlow:
    def __init__(self, order, log=False):
        self.order = order
        self.log = log
        self.logger = Logger(log)

    def start(self):
        result = {}

        flow_input = None
        flow_input_type = None

        for index, task in enumerate(self.order, start=1):

            # log
            self.logger.log_head(f"- Executing task: {task.desc}")

            task.execute(flow_input, flow_input_type)

            if not task.exclude:
                result[f'task{index}'] = task.output

            # define the input for next step
            flow_input = task.output
            flow_input_type = task.output_type

        return result



from enum import Enum


class AgentTypes(Enum):
    TEXT = 'text'
    IMAGE = 'image'
    VISION = 'vision'


class InputTypes(Enum):
    TEXT = 'text'
    IMAGE = 'image'
    VISION = 'vision'

class Matcher():
    input = {
        'text': 'text',
        'image': 'text',
        'vision': 'image'
    }

    output = {
        'text': 'text',
        'image': 'image',
        'vision': 'text'
    }


from abc import ABC, abstractmethod


class Template(ABC):

    @abstractmethod
    def apply_input(self, data):
        pass

    @abstractmethod
    def apply_output(self, data):
        pass


class TextInputTemplate(Template):

    def __init__(self, template_text: str, previous_input_tag='context', user_request_tag='user request'):
        if '{0}' not in template_text:
            context = previous_input_tag + ': {0}\n'
            request = user_request_tag + ': ' + template_text
            template_text = context + request

        self.template_text = template_text.strip()

    def apply_input(self, data):
        return self.template_text.format(data)

    def apply_output(self, data):
        pass



import unittest
from intelli.wrappers.googleai_wrapper import GoogleAIWrapper
import os 
from dotenv import load_dotenv
load_dotenv()

class TestGoogleAIWrapper(unittest.TestCase):

    def setUp(self):
        self.api_key = os.getenv('GOOGLE_API_KEY')
        self.googleai = GoogleAIWrapper(self.api_key)
   
    def test_generate_speech(self):
        params = {
            "text": 'Welcome to IntelliNode',
            "languageCode": 'en-US',
            "name": 'en-US-Wavenet-A',
            "ssmlGender": 'MALE',
        }

        result = self.googleai.generate_speech(params)
        print('Generate Speech Result:', result)
        self.assertTrue(len(result) > 0)

if __name__ == "__main__":
    unittest.main()



import unittest
from intelli.function.chatbot import Chatbot
from intelli.model.input.chatbot_input import ChatModelInput
import os
from dotenv import load_dotenv
load_dotenv()

class TestOpenAIWrapper(unittest.TestCase):

    def setUp(self):
        self.api_key = os.getenv('OPENAI_API_KEY')
        self.one_key = os.getenv('CUSTOM_ONE')
        
    def test_multiple_choice(self):
        
        result = gen_10q(self.api_key, self.one_key, "Power supply systems")
        
        print('the model response: \n', result)
        
        self.assertTrue(len(result) > 0, "The result is empty")


def gen_10q(api, one, subject):
    # Creating chatbot with the intellinode one key:
    bot = Chatbot(api, "openai", {"one_key": one})

    input = ChatModelInput("Generate 10 multiple choice questions from the context.", "gpt-4-turbo-preview")
    input.add_user_message(f"Generate 10 multiple choice questions about {subject} from the context, and let me know the correct choice for each.")

    # Optional to return the searched file name:
    input.attach_reference = True

    response = bot.chat(input)
    print(f"Reference used: {response['references']}")
    return response['result'][0]



if __name__ == "__main__":
    unittest.main()


import requests

from intelli.config import config
from intelli.utils.conn_helper import ConnHelper


class CohereAIWrapper:
    def __init__(self, api_key):
        self.API_BASE_URL = config['url']['cohere']['base']
        self.COHERE_VERSION = config['url']['cohere']['version']
        self.API_KEY = api_key
        self.headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {self.API_KEY}',
            'Cohere-Version': self.COHERE_VERSION,
        }

    def generate_text(self, params):
        url = config['url']['cohere']['completions']
        try:
            response = requests.post(f'{self.API_BASE_URL}{url}', json=params, headers=self.headers)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def generate_chat_text(self, params):
        url = config['url']['cohere']['chat']
        try:
            response = requests.post(f'{self.API_BASE_URL}{url}', json=params, headers=self.headers)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def get_embeddings(self, params):
        url = config['url']['cohere']['embed']
        try:
            response = requests.post(f'{self.API_BASE_URL}{url}', json=params, headers=self.headers)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))



config = {
    "url": {
        "intellicloud": {
            "base": "https://ue8sdr9bij.execute-api.us-east-2.amazonaws.com/v1",
            "semantic_search": "/semantic_search/"
        },
        "openai": {
            "base": "https://api.openai.com",
            "completions": "/v1/completions",
            "chatgpt": "/v1/chat/completions",
            "imagegenerate": "/v1/images/generations",
            "embeddings": "/v1/embeddings",
            "audiotranscriptions": "/v1/audio/transcriptions",
            "audiospeech": "/v1/audio/speech",
            "files": "/v1/files",
            "finetuning": "/v1/fine_tuning/jobs",
            "organization": None
        },
        "azure_openai": {
            "base": "https://{resource-name}.openai.azure.com/openai",
            "completions": "/deployments/{deployment-id}/completions?api-version={api-version}",
            "chatgpt": "/deployments/{deployment-id}/chat/completions?api-version={api-version}",
            "imagegenerate": "/images/generations:submit?api-version={api-version}",
            "embeddings": "/deployments/{deployment-id}/embeddings?api-version={api-version}",
            "audiotranscriptions": "/deployments/{deployment-id}/audio/transcriptions?api-version={api-version}",
            "audiospeech": "/deployments/{deployment-id}/audio/speech?api-version={api-version}",
            "files": "/files?api-version={api-version}",
            "finetuning": "/fine_tuning/jobs?api-version={api-version}"
        },
        "cohere": {
            "base": "https://api.cohere.ai",
            "completions": "/generate",
            "chat": "/chat",
            "embed": "/v1/embed",
            "version": "2022-12-06"
        },
        "google": {
            "base": "https://{0}.googleapis.com/v1",
            "speech": {
                "prefix": "texttospeech",
                "synthesize": {
                    "postfix": "/text:synthesize"
                }
            }
        },
        "stability": {
            "base": "https://api.stability.ai",
            "text_to_image": "/v1/generation/{0}/text-to-image",
            "upscale": "/v1/generation/{0}/image-to-image/upscale",
            "image_to_image": "/v1/generation/{0}/image-to-image"
        },
        "huggingface": {
            "base": "https://api-inference.huggingface.co/models"
        },
        "replicate": {
            "base": "https://api.replicate.com",
            "predictions": "/v1/predictions"
        },
        "mistral": {
            "base": "https://api.mistral.ai",
            "completions": "/v1/chat/completions",
            "embed": "/v1/embeddings"
        },
        "gemini": {
            "base": "https://generativelanguage.googleapis.com/v1beta/models",
            "contentEndpoint": "/gemini-pro:generateContent",
            "visionEndpoint": "/gemini-pro-vision:generateContent",
            "embeddingEndpoint": "/embedding-001:embedContent",
            "batchEmbeddingEndpoint": "/embedding-001:batchEmbedContents"
        }
    },
    "models": {
        "replicate": {
            "llama": {
                "70b": "70b-chat",
                "13b": "13b-chat",
                "70b-chat": "70b-chat",
                "13b-chat": "13b-chat",
                "34b-code": "34b-code",
                "34b-python": "34b-python",
                "13b-code-instruct": "13b-code-instruct",
                "llama-2-13b-embeddings": "llama-2-13b-embeddings",
                "70b-chat-version": "02e509c789964a7ea8736978a43525956ef40397be9033abf9fd2badfe68c9e3",
                "13b-chat-version": "f4e2de70d66816a838a89eeeb621910adffb0dd0baba3976c96980970978018d",
                "34b-code-version": "efbd2ef6feefb242f359030fa6fe08ce32bfced18f3868b2915db41d41251b46",
                "34b-python-version": "482ba325daab209d121f45a0030f2f3ed942df98b185d41635ab3f19165a3547",
                "13b-code-instruct-version": "ca8c51bf3c1aaf181f9df6f10f31768f065c9dddce4407438adc5975a59ce530",
                "llama-2-13b-embeddings-version": "7115a4c65b86815e31412e53de1211c520164c190945a84c425b59dccbc47148"
            }
        }
    }
}


import unittest
import os
from intelli.controller.remote_vision_model import RemoteVisionModel
from intelli.model.input.vision_input import VisionModelInput
from dotenv import load_dotenv

load_dotenv()

class TestRemoteVisionModel(unittest.TestCase):
    
    def setUp(self):
        self.openai_api_key = os.getenv('OPENAI_API_KEY')
        self.gemini_api_key = os.getenv('GEMINI_API_KEY')

        if not self.openai_api_key or not self.gemini_api_key:
            raise unittest.SkipTest("Both OpenAI and Gemini keys are required for testing RemoteVisionModel")
    
    def test_openai_image_descriptor(self):
        print('--- call openai vision ---')
        provider = "openai"
        controller = RemoteVisionModel(self.openai_api_key, provider)

        vision_input = VisionModelInput(content = "Describe the image", file_path = '../temp/test_image_desc.png', model = "gpt-4-vision-preview")
        result = controller.image_to_text(vision_input)
        
        print(result)
    
    def test_gemini_image_descriptor(self):
        print('--- call gemini vision ---')
        provider = "gemini"
        controller = RemoteVisionModel(self.gemini_api_key, provider)

        vision_input = VisionModelInput(content = "Describe this image", file_path = '../temp/test_image_desc.png', extension='png')
        result = controller.image_to_text(vision_input)
        
        print(result)

if __name__ == '__main__':
    unittest.main()



import os
import base64
import unittest
from intelli.flow.types import *
from intelli.flow.agents.agent import Agent
from intelli.flow.input.task_input import TextTaskInput, ImageTaskInput
from intelli.flow.processors.basic_processor import TextProcessor
from intelli.flow.sequence_flow import SequenceFlow
from intelli.flow.tasks.task import Task
from dotenv import load_dotenv


load_dotenv()


class TestFlows(unittest.TestCase):
    def setUp(self):
        # Initiate the keys
        self.openai_api_key = os.getenv("OPENAI_API_KEY")
        self.gemini_key = os.getenv("GEMINI_API_KEY")
        self.stability_key = os.getenv("STABILITY_API_KEY")
    
    def test_blog_post_flow(self):
        print("---- start blog portal flow ----")
        
        # Define agents
        blog_agent = Agent(
            agent_type=AgentTypes.TEXT.value,
            provider="openai",
            mission="write blog posts",
            model_params={"key": self.openai_api_key, "model": "gpt-3.5-turbo"},
        )
        description_agent = Agent(
            agent_type=AgentTypes.TEXT.value,
            provider="gemini",
            mission="generate description only",
            model_params={"key": self.gemini_key, "model": "gemini"},
        )
        image_agent = Agent(
            agent_type=AgentTypes.IMAGE.value,
            provider="stability",
            mission="generate image",
            model_params={"key": self.stability_key},
        )

        # Define tasks
        task1 = Task(
            TextTaskInput("blog post about electric cars"), blog_agent, log=True
        )
        task2 = Task(
            TextTaskInput("Write short image description for image generation model"),
            description_agent,
            pre_process=TextProcessor.text_head,
            log=True,
        )
        task3 = Task(
            TextTaskInput("Generate cartoon style image"), image_agent, exclude=True, log=True
        )

        # Start SequenceFlow
        flow = SequenceFlow([task1, task2, task3], log=True)
        final_result = flow.start()

        print("Final result:", final_result)
    
    def test_flow_chart_image_flow(self):
        print("---- start vision coder flow ----")
        
        analyst = Agent(
            agent_type=AgentTypes.VISION.value,
            provider="openai",
            mission="describe flow charts from images",
            model_params={"key": self.openai_api_key, "extension": "jpg", "model": "gpt-4-vision-preview"},
        )
        
        coder = Agent(
            agent_type=AgentTypes.TEXT.value,
            provider="openai",
            mission="write python code. response only with the code without explination or text or marks.",
            model_params={"key": self.openai_api_key, "model": "gpt-3.5-turbo"},
        )
        
        # Define tasks
        with open('../temp/code_flow_char.jpg', "rb") as image_file:
            image_data = base64.b64encode(image_file.read()).decode('utf-8')
                
        task1 = Task(
            ImageTaskInput(desc="describe the steps of the code flow chat for an engineer.", img=image_data), agent=analyst, log=True
        )

        task2 = Task(
            TextTaskInput("write python code from the provided context"), agent=coder, log=True
        )
        
        # Start SequenceFlow
        flow = SequenceFlow([task1, task2], log=True)
        final_result = flow.start()

        print("Final result:", final_result)

if __name__ == "__main__":
    unittest.main()



import os
import requests

from intelli.config import config
from intelli.utils.conn_helper import ConnHelper


class StabilityAIWrapper:
    def __init__(self, api_key):
        self.api_base_url = config['url']['stability']['base']
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {api_key}',
            'Accept': 'application/json',
        })

    def generate_images(self, params, engine='stable-diffusion-xl-1024-v1-0'):
        endpoint = config['url']['stability']['text_to_image'].format(engine)
        url = f"{self.api_base_url}{endpoint}"
        try:
            response = self.session.post(url, json=params)
            response.raise_for_status()
            return response.json()
        except requests.exceptions.RequestException as e:
            raise Exception(ConnHelper.get_error_message(e))



import base64
import os
import unittest
from dotenv import load_dotenv
from intelli.controller.remote_speech_model import RemoteSpeechModel, SupportedSpeechModels
from intelli.model.input.text_speech_input import Text2SpeechInput

load_dotenv()

class TestRemoteSpeechModel(unittest.TestCase):

    def setUp(self):
        self.api_key_google = os.getenv('GOOGLE_API_KEY')
        self.api_key_openai = os.getenv('OPENAI_API_KEY')
        self.remote_speech_model_google = RemoteSpeechModel(self.api_key_google, SupportedSpeechModels['GOOGLE'])
        self.remote_speech_model_openai = RemoteSpeechModel(self.api_key_openai, SupportedSpeechModels['OPENAI'])
        self.temp_dir = '../temp'

        if not os.path.exists(self.temp_dir):
            os.makedirs(self.temp_dir)

    def test_generate_speech_google(self):
        input_params = Text2SpeechInput('Welcome to Intellinode.', 'en-gb')
        audio_content = self.remote_speech_model_google.generate_speech(input_params)
        self.assertTrue(audio_content, "audio_content should not be None")
        
        # eecode the base64
        audio_data = base64.b64decode(audio_content)
        google_file_path = os.path.join(self.temp_dir, 'google_speech.mp3')
        
        # save
        with open(google_file_path, 'wb') as audio_file:
            audio_file.write(audio_data)
        
        self.assertTrue(os.path.exists(google_file_path), "Google TTS MP3 file should be saved")

    def test_generate_speech_openai(self):
        input_params = Text2SpeechInput('Welcome to Intellinode.', 'en-US', 'MALE', 'alloy', 'tts-1', True)
        result = self.remote_speech_model_openai.generate_speech(input_params)
        self.assertTrue(result, "result should not be None")
        
        # write the streaming audio
        openai_file_path = os.path.join(self.temp_dir, 'openai_speech.mp3')
        with open(openai_file_path, 'wb') as audio_file:
            for chunk in result:
                audio_file.write(chunk)
        
        self.assertTrue(os.path.exists(openai_file_path), "OpenAI TTS MP3 file should be saved")

if __name__ == "__main__":
    unittest.main()



import os
import requests
from urllib.parse import urljoin

from intelli.utils.conn_helper import ConnHelper
from intelli.utils.proxy_helper import ProxyHelper


class OpenAIWrapper:

    def __init__(self, api_key, proxy_helper=None):
        self.api_key = api_key
        self.proxy_helper = proxy_helper or ProxyHelper.get_instance()
        # set the headers
        if self.proxy_helper.get_openai_type() == 'azure':
            print('Set OpenAI Azure settings')
            if not self.proxy_helper.get_openai_resource_name():
                raise ValueError('Set your Azure resource name')
            headers = {
                'Content-Type': 'application/json',
                'api-key': f'{self.api_key}',
            }
        else:
            headers = {
                'Content-Type': 'application/json',
                'Authorization': f'Bearer {self.api_key}',
            }
            # check if organization exists for non-Azure
            organization = self.proxy_helper.get_openai_organization()
            if organization:
                headers['OpenAI-Organization'] = organization

        # define the connection session
        self.session = BaseURLSession(base_url=self.proxy_helper.get_openai_url())
        self.session.headers.update(headers)

    def generate_chat_text(self, params, functions=None, function_call=None):
        url = self.proxy_helper.get_openai_chat_url(params['model'])
        payload = params.copy()
        if functions:
            payload['functions'] = functions
        if function_call:
            payload['function_call'] = function_call

        try:
            response = self.session.post(url, json=payload, stream=params.get('stream', False))
            response.raise_for_status()
            if params.get('stream', False):
                return response.iter_lines(decode_unicode=True)
            else:
                return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))
        finally:
            self.session.close()

    def generate_images(self, params):
        url = self.proxy_helper.get_openai_image_url()
        try:
            response = self.session.post(url, json=params)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))
        finally:
            self.session.close()

    def upload_file(self, file_path, purpose):

        url = urljoin(self.proxy_helper.openai_url, self.proxy_helper.get_openai_files_url())

        with open(file_path, 'rb') as file:
            files = {
                'file': (os.path.basename(file_path), file, 'application/jsonl')
            }
            data = {'purpose': purpose}
            headers = {
                'Authorization': f'Bearer {self.api_key}'
            }

            # make direct post request due to conflicts in common content type
            response = requests.post(url, headers=headers, files=files, data=data)
            response.raise_for_status()
            return response.json()

    def store_fine_tuning_data(self, params):
        url = self.proxy_helper.get_openai_finetuning_job_url()
        try:
            response = self.session.post(url, json=params)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def list_fine_tuning_data(self):
        url = self.proxy_helper.get_openai_finetuning_job_url()
        try:
            response = self.session.get(url)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def get_embeddings(self, params):
        url = self.proxy_helper.get_openai_embed_url(params['model'])
        try:
            response = self.session.post(url, json=params)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def speech_to_text(self, params, headers=None):
        url = self.proxy_helper.get_openai_audio_transcriptions_url(params.get('model', ''))
        try:
            custom_headers = self.session.headers.copy()
            if headers:
                custom_headers.update(headers)
            response = self.session.post(url, data=params, headers=custom_headers)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def text_to_speech(self, params, headers=None):
        url = self.proxy_helper.get_openai_audio_speech_url(params['model'])
        try:
            custom_headers = self.session.headers.copy()
            if headers:
                custom_headers.update(headers)
            response = self.session.post(url, json=params, headers=custom_headers, stream=True)
            response.raise_for_status()
            return response.iter_content(chunk_size=8192) if params.get('stream', False) else response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))

    def image_to_text(self, params, headers=None):
        url = self.proxy_helper.get_openai_chat_url(params['model'])

        if headers:
            combined_headers = {**self.session.headers, **headers}
        else:
            combined_headers = self.session.headers

        try:
            response = self.session.post(url, json=params, headers=combined_headers)
            response.raise_for_status()
            return response.json()
        except requests.RequestException as error:
            raise Exception(ConnHelper.get_error_message(error))


class BaseURLSession(requests.Session):
    def __init__(self, base_url=None, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.base_url = base_url

    def request(self, method, url, *args, **kwargs):
        # Prepend the base URL if it's defined and the URL is relative
        if self.base_url and not url.startswith(("http://", "https://")):
            url = f"{self.base_url}{url}"
        return super().request(method, url, *args, **kwargs)



import unittest
import os
from intelli.wrappers.stability_wrapper import StabilityAIWrapper
import base64
from dotenv import load_dotenv
load_dotenv()

class TestStabilityAIWrapper(unittest.TestCase):
    
    def setUp(self):
        # Ensure the STABILITY_API_KEY environment variable is set
        self.api_key = os.getenv('STABILITY_API_KEY')
        if self.api_key is None:
            raise unittest.SkipTest("STABILITY_API_KEY environment variable is not set")
        
        self.wrapper = StabilityAIWrapper(self.api_key)

    def test_generate_text_to_image(self):
        print('start genering image')
        # Define the parameters for the text to image generation
        params = {
            "text_prompts": [
                {
                    "text": "A quaint cottage in a forest clearing, under a starry night sky"
                }
            ],
            "cfg_scale": 7,
            "height": 1024,
            "width": 1024,
            "samples": 1,
            "steps": 20
        }

        result = self.wrapper.generate_images(params)

        # Verify the response contains expected keys
        self.assertIn('artifacts', result)
        self.assertTrue(isinstance(result['artifacts'], list), "Artifacts should be a list")
        self.assertTrue(len(result['artifacts']) > 0, "Artifacts list should not be empty")
        artifact = result['artifacts'][0]
        self.assertIn('base64', artifact, "Artifact should contain a base64 key")
        
        # Decode base64 and save the image
        image_data = base64.b64decode(artifact['base64'])

        output_dir = '../temp'
        os.makedirs(output_dir, exist_ok=True)
        output_path = os.path.join(output_dir, 'stability_generate.png')

        with open(output_path, 'wb') as img_file:
            img_file.write(image_data)
        
        print(f"Saved generated image to {output_path}")

if __name__ == '__main__':
    unittest.main()


import base64
import os
import unittest
from dotenv import load_dotenv

from intelli.flow.agents.agent import Agent
from intelli.flow.input.task_input import TextTaskInput, ImageTaskInput
from intelli.flow.processors.basic_processor import TextProcessor
from intelli.flow.sequence_flow import SequenceFlow
from intelli.flow.tasks.task import Task
from intelli.flow.types import *

load_dotenv()


class TestFlows(unittest.TestCase):
    def setUp(self):
        # Initiate the keys
        self.openai_api_key = os.getenv("OPENAI_API_KEY")
        self.gemini_key = os.getenv("GEMINI_API_KEY")
        self.stability_key = os.getenv("STABILITY_API_KEY")

    def test_icon_generate_flow(self):
        print("---- start icons flow ----")

        # Define agents
        desc_agent = Agent(
            agent_type=AgentTypes.TEXT.value,
            provider="openai",
            mission="generate image description from the user input to use it for DALL·E icon generation",
            model_params={"key": self.openai_api_key, "model": "gpt-3.5-turbo"},
        )

        image_agent = Agent(
            agent_type=AgentTypes.IMAGE.value,
            provider="openai",
            mission="generate image",
            model_params={"key": self.openai_api_key, "model": "dall-e-3", "width": 1024, "height": 1024},
        )

        # Define tasks
        task2 = Task(
            TextTaskInput("flat icon about {0}"), image_agent, log=False
        )

        task1_list = []
        topics = ["unified ai models access", "evaluate large language models", "workflows"]

        for topic in topics:
            task1 = Task(
                TextTaskInput(
                    "Write simple icon description cartoon style inspired from docusaurus style about: {}".format(
                        topic)),
                desc_agent,
                log=False,
            )
            task1_list.append(task1)

        # Start SequenceFlow
        for index, task1 in enumerate(task1_list):
            print(f'---- Execute task {index+1} ----')
            flow = SequenceFlow([task1, task2], log=False)
            final_result = flow.start()

            print(f"{index + 1}- flow result:", final_result)


if __name__ == "__main__":
    unittest.main()



from intelli.model.input.text_speech_input import Text2SpeechInput
from intelli.wrappers.googleai_wrapper import GoogleAIWrapper
from intelli.wrappers.openai_wrapper import OpenAIWrapper

SupportedSpeechModels = {
    'GOOGLE': 'google',
    'OPENAI': 'openAi',
}


class RemoteSpeechModel:

    def __init__(self, key_value, provider=None):
        if not provider:
            provider = SupportedSpeechModels['GOOGLE']

        supported_models = self.get_supported_models()

        if provider in supported_models:
            self.initiate(key_value, provider)
        else:
            models = " - ".join(supported_models)
            raise ValueError(f"The received key value is not supported. Send any model from: {models}")

    def initiate(self, key_value, key_type):
        self.key_type = key_type
        if key_type == SupportedSpeechModels['GOOGLE']:
            self.google_wrapper = GoogleAIWrapper(key_value)
        elif key_type == SupportedSpeechModels['OPENAI']:
            self.openai_wrapper = OpenAIWrapper(key_value)
        else:
            raise ValueError('Invalid provider name')

    def get_supported_models(self):
        return list(SupportedSpeechModels.values())

    def generate_speech(self, input_params):
        if not isinstance(input_params, Text2SpeechInput):
            raise ValueError('Invalid input: Must be an instance of Text2SpeechInput')

        if self.key_type == SupportedSpeechModels['GOOGLE']:
            params = input_params.get_google_input()
            response = self.google_wrapper.generate_speech(params)
            return response

        elif self.key_type == SupportedSpeechModels['OPENAI']:
            params = input_params.get_openai_input()
            response = self.openai_wrapper.text_to_speech(params)
            return response
        else:
            raise ValueError('The keyType is not supported')



import json

from intelli.model.input.chatbot_input import ChatModelInput
from intelli.utils.system_helper import SystemHelper
from intelli.wrappers.geminiai_wrapper import GeminiAIWrapper
from intelli.wrappers.intellicloud_wrapper import IntellicloudWrapper
from intelli.wrappers.mistralai_wrapper import MistralAIWrapper
from intelli.wrappers.openai_wrapper import OpenAIWrapper


class Chatbot:

    def __init__(self, api_key, provider, options=None):
        if options is None:
            options = {}
        self.api_key = api_key
        self.provider = provider.lower()
        self.options = options
        self.wrapper = self._initialize_provider()
        self.extended_search = IntellicloudWrapper(options['one_key'],
                                                   options.get('api_base', None)) if 'one_key' in options else None
        self.system_helper = SystemHelper()

    def _initialize_provider(self):
        if self.provider == 'openai':
            proxy_helper = self.options.get('proxy_helper', None)
            return OpenAIWrapper(self.api_key, proxy_helper=proxy_helper)
        elif self.provider == 'mistral':
            return MistralAIWrapper(self.api_key)
        elif self.provider == 'gemini':
            return GeminiAIWrapper(self.api_key)
        else:
            raise ValueError(f"Unsupported provider: {self.provider}")

    def chat(self, chat_input):
        if not isinstance(chat_input, ChatModelInput):
            raise TypeError("chat_input must be an instance of ChatModelInput")

        references = []
        if self.extended_search:
            references = self._augment_with_semantic_search(chat_input)

        get_input_method = f"get_{self.provider}_input"
        chat_method = getattr(self, f"_chat_{self.provider}", None)
        if not chat_method:
            raise NotImplementedError(f"{self.provider.capitalize()} chat is not implemented.")

        params = getattr(chat_input, get_input_method)()
        result = chat_method(params)
        return {'result': result, 'references': references} if chat_input.attach_reference else result

    def _chat_openai(self, params):
        results = self.wrapper.generate_chat_text(params)
        return self._parse_openai_responses(results)

    def _chat_mistral(self, params):
        response = self.wrapper.generate_text(params)
        return [choice['message']['content'] for choice in response.get('choices', [])]

    def _chat_gemini(self, params):
        response = self.wrapper.generate_content(params)
        output = []
        for candidate in response.get("candidates", []):
            if "content" in candidate:
                output.append(candidate["content"]["parts"][0]["text"])
            else:
                raise Exception("Error when calling gemini: {}".format(response))
        return output

    def stream(self, chat_input):
        """
        Streams responses from OpenAI for the given chat input.
        
        Each yielded content is the text content alone, extracted from the streamed response.
        """
        if self.provider != 'openai':
            raise NotImplementedError("Streaming is only supported for OpenAI.")

        if self.extended_search:
            _ = self._augment_with_semantic_search(chat_input)

        params = chat_input.get_openai_input()

        for content in self._stream_openai(params):
            yield content

    def _stream_openai(self, params):
        """
        Private helper method to stream text from OpenAI and parse each content chunk.
        """
        params['stream'] = True
        for response in self.wrapper.generate_chat_text(params):
            if response.strip() and response.startswith('data: ') and response != 'data: [DONE]':
                json_content = response[len('data: '):].strip()

                try:
                    data_chunk = json.loads(json_content)
                    content = data_chunk.get("choices", [{}])[0].get("delta", {}).get("content", "")
                    if content:
                        yield content
                except json.JSONDecodeError as e:
                    print("Error decoding JSON:", e)

    # helpers
    def _parse_openai_responses(self, results):
        responses = []
        for choice in results.get('choices', []):
            response = choice.get('message', {}).get('content', '')
            if choice.get('finish_reason') == 'function_call' and 'function_call' in choice.get('message', {}):
                response['function_call'] = choice['message']['function_call']
            responses.append(response)
        return responses

    def _augment_with_semantic_search(self, chat_input):
        last_user_message = chat_input.messages[-1].content if chat_input.messages else ""
        references = []
        if last_user_message:
            # Perform the semantic search based on the last user message.
            search_results = self.extended_search.semantic_search(last_user_message, chat_input.search_k)

            # Accumulate document names from the search results for references.
            references = {}
            for doc in search_results:
                doc_name = doc['document_name']
                if doc_name not in references:
                    references[doc_name] = {'pages': []}
                # Assuming each 'doc' can contain multiple 'pages' or similar structures, adjust as necessary.
                references[doc_name]['pages'].extend(doc.get('pages', []))

            # Generate context data based on the semantic search results.
            context_data = '\n'.join(
                data['text'] for doc in search_results for data in doc['data'] if 'text' in data
            ).strip()

            # Load the static prompt template for an augmented chatbot response.
            augmented_message_template = self.system_helper.load_static_prompt("augmented_chatbot")
            augmented_message = augmented_message_template.replace("${semantic_search}", context_data).replace(
                "${user_query}", last_user_message)

            # Replace the content of the last user message with the augmented message in the ChatModelInput.
            chat_input.messages[-1].content = augmented_message

        return references
